@@ -1,4 +1,4 @@
-;;;;  Copyright (C) 2022 Andrea G. Monaco
+;;;;  Copyright (C) 2022-2023 Andrea G. Monaco
 
 ;;;;  This file is part of alisp, a lisp implementation.
 
@@ -17,6 +17,11 @@
 
 
 
+(in-package cl)
+
+
+
+
 ;;; the following useless constants are just ridiculously high because there is no actual limit
 (defconstant LAMBDA-PARAMETERS-LIMIT 4294967296)
 (defconstant CALL-ARGUMENTS-LIMIT 4294967296)
@@ -154,18 +159,24 @@
 
 
 (defmacro and (&rest forms)
-  `(if ,(not forms)
-       t
-       (if ,(car forms)
-	   (and ,@(cdr forms)))))
+  `(let ((val (multiple-value-list ,(car forms))))
+     (if ,(not forms)
+         t
+         (if (car val)
+             (if ,(not (cdr forms))
+                 (values-list val)
+                 (and ,@(cdr forms)))))))
 
 
 (defmacro or (&rest forms)
-  `(if ,(not forms)
+  `(let ((val (multiple-value-list ,(car forms))))
+     (if ,(not forms)
        nil
-       (if ,(car forms)
-	   t
-	   (or ,@(cdr forms)))))
+       (if (car val)
+	   (if ,(not (cdr forms))
+               (values-list val)
+	       (car val))
+           (or ,@(cdr forms))))))
 
 
 (defmacro cond (&body body)
@@ -314,6 +325,22 @@
   (if (member obj list) list (cons obj list)))
 
 
+(defun fill (seq obj &key (start 0) end)
+  (dotimes (i (if end (- end start) (length seq)))
+    (setf (elt seq (+ i start)) obj))
+  seq)
+
+
+(defmacro push (item place)
+  `(setf ,place (cons ,item ,place)))
+
+
+(defmacro pop (place)
+  `(let ((c (car ,place)))
+     (setf ,place (cdr ,place))
+     c))
+
+
 (defun array-rank (array)
   (length (array-dimensions array)))
 
@@ -490,7 +517,15 @@
   (cond
     ((numberp x) (eql x y))
     ((characterp x) (eql x y))
-    ((consp x) (and (equal (car x) (car y)) (equal (cdr x) (cdr y))))
+    ((and (consp x) (consp y))
+     (and
+      (let (l)
+	(= (setq l (length x)) (length y))
+	(dotimes (i l)
+	  (if (equal (car x) (car y))
+	      (setq x (cdr x) y (cdr y))
+	      (return-from equal nil)))
+	(equal x y))))
     (t (eq x y))))
 
 
@@ -498,12 +533,26 @@
   (cond
     ((and (numberp x) (numberp y)) (= x y))
     ((and (characterp x) (characterp y)) (char-equal x y))
-    ((consp x) (and (equalp (car x) (car y)) (equalp (cdr x) (cdr y))))
-    ((and (arrayp x) (arrayp y)) (and
-				  (equal (array-dimensions x) (array-dimensions y))
-				  (dotimes (i (array-total-size x) t)
-				    (unless (equalp (row-major-aref x i) (row-major-aref y i))
-				      (return-from equalp nil)))))
+    ((and (consp x) (consp y))
+     (and
+      (let (l)
+	(= (setq l (length x)) (length y))
+	(dotimes (i l)
+	  (if (equalp (car x) (car y))
+	      (setq x (cdr x) y (cdr y))
+	      (return-from equalp nil)))
+	(equalp x y))))
+    ((and (arrayp x) (arrayp y)) (let ((dimsx (array-dimensions x))
+				       (dimsy (array-dimensions y)))
+				   (and
+				    (= (length dimsx) (length dimsy))
+				    (dolist (dimx dimsx t)
+				      (unless (= dimx (car dimsy))
+					 (return-from equalp nil))
+				      (setq dimsy (cdr dimsy)))
+				    (dotimes (i (array-total-size x) t)
+				      (unless (equalp (row-major-aref x i) (row-major-aref y i))
+					(return-from equalp nil))))))
     (t (eq x y))))
 
 
@@ -535,15 +584,28 @@
   string)
 
 
-(defun prin1 (obj)
+(defun write-sequence (seq str &key (start 0) end)
+  (dotimes (i (if end (- end start) (length seq)))
+    (princ (elt seq (+ i start)) str))
+  seq)
+
+
+(defun prin1 (obj &optional (out *standard-output*))
   (let ((*print-escape* t))
-    (write obj)))
+    (write obj :stream out)))
 
 
-(defun princ (obj)
+(defun princ (obj &optional (out *standard-output*))
   (let ((*print-escape* nil)
 	(*print-readably* nil))
-    (write obj)))
+    (write obj :stream out)))
+
+
+(defun print (obj &optional (out *standard-output*))
+  (terpri out)
+  (prin1 obj out)
+  (write-char #\space out)
+  obj)
 
 
 (defun format (out fstr &rest args)
@@ -563,3 +625,31 @@
 	    (if (char= ch #\~)
 		(setq in-spec t at-sign nil colon nil sign nil num nil dirargs nil)
 		(write-char ch)))))))
+
+
+
+
+(export '(lambda-parameters-limit call-arguments-limit multiple-values-limit
+	  array-rank-limit array-dimension-limit array-total-size-limit
+	  char-code-limit identity pi 1+ 1- minusp plusp abs zerop mod rem evenp
+	  oddp isqrt conjugate *gensym-counter* gensym first second third fourth
+	  fifth sixth seventh eighth ninth tenth rest caar cadr cdar cddr caaar
+	  caadr cadar caddr cdaar cdadr cddar cdddr caaaar caaadr caadar caaddr
+	  cadaar cadadr caddar cadddr cdaaar cdaadr cdadar cdaddr cddaar cddadr
+	  cdddar cddddr endp when unless incf decf and or cond return every some
+	  notany notevery member member-if member-if-not find find-if
+	  find-if-not assoc assoc-if assoc-if-not position position-if
+	  position-if-not count count-if count-if-not remove remove-if-not
+	  delete delete-if delete-if-not nreverse adjoin fill push pop
+	  array-rank array-dimension array-total-size array-in-bounds-p char
+	  string/= char-equal digit-char digit-char-p char-int string-upcase
+	  string-downcase string-capitalize nstring-upcase nstring-downcase
+	  nstring-capitalize consp listp symbolp keywordp functionp packagep
+	  integerp rationalp floatp complexp characterp vectorp arrayp sequencep
+	  stringp hash-table-p pathnamep streamp realp numberp equal equalp
+	  fdefinition complement mapc terpri write-line write-sequence prin1
+	  princ print format))
+
+
+
+(in-package cl-user)
