@@ -1,7 +1,7 @@
 #!/usr/bin/perl -w
 use strict;
 
-# Copyright (C) 2022 Andrea G. Monaco
+# Copyright (C) 2022-2023 Andrea G. Monaco
 # 
 # This file is part of alisp, a lisp implementation.
 #
@@ -65,8 +65,10 @@ make_test ("'(\n" .
 	   ")\n" .
 	   ")", "(NIL)");
 make_test ("\"\"", "\"\"");
+make_test ("\"\" \"\" \"\"", "\"\"\n\"\"\n\"\"");
 make_test ("` ( ; asd\n" .
            " \"\")", "(\"\")");
+make_test ("`';\na", "'A");
 make_test ("#|\n" .
 	   "|# \"\"", "\"\"");
 make_test ("#|\n" .
@@ -88,9 +90,19 @@ make_test ("'(1 . (\n" .
 	   "2))", "(1 2)");
 make_test ("'( \"\" #||# )", "(\"\")");
 make_test ("'asd\\f|gh|j", "|ASDfghJ|");
+make_test ("'\\\\", "\\\\");
+make_test ("'\\123", "|123|");
+make_test ("'123\\ ", "|123 |");
+make_test ("'\\.", "|.|");
+make_test ("'\\..", "|..|");
 make_test ("'\\\n ", "|\n|");
 make_test ("'a\\\n ", "|A\n|");
+make_test ("'|aaa\nbb|c", "|aaa\nbbC|");
+make_test ("'a\\\n..", "|A\n..|");
+make_test ("'|aaa\n:bbb|", "|aaa\n:bbb|");
 make_test (":\\asd\\\\f", ":|aSD\\\\F|");
+make_test ("'cl:car", "CAR");
+make_test ("'cl-user::car", "CAR");
 
 
 # now we extensively test backtick notation.  maybe some of these
@@ -106,6 +118,7 @@ make_test ("``(a ,,(+ 1 2) ,(+ 3 4))", "`(A ,3 ,(+ 3 4))");
 make_test ("``(a ,(+ ,1 2))", "`(A ,(+ 1 2))");
 make_test ("`(,1 . ,2)", "(1 . 2)");
 make_test ("`(,1 ,2 . ,3)", "(1 2 . 3)");
+make_test ("`',(car ())", "'NIL");
 make_test ("`(if ``(progn ,,1))", "(IF ``(PROGN ,,1))");
 make_test ("'@", "@");
 make_test ("`(,@())", "NIL");
@@ -139,7 +152,9 @@ make_test ("#\\newLINE", "#\\Newline");
 make_test ("#\\\n ", "#\\Newline");
 make_test ("#\\Newline", "#\\Newline");
 make_test ("#.\"\"", "\"\"");
+make_test ("#. (+ 1 2)", "3");
 make_test ("#p\".\"", "#P\".\"");
+make_test ("#(a b c)", "#(A B C)");
 make_test ("#(1 2 3)", "#(1 2 3)");
 make_test ("#(1 2\n" .
 	   "3)", "#(1 2 3)");
@@ -199,11 +214,14 @@ make_test ("(nconc (list 'a 'b 'c) 'd)", "(A B C . D)");
 make_test ("(nconc (list 'a 'b 'c) NIL (list 'd 'e) NIL 'f)", "(A B C D E . F)");
 make_test ("(let ((a 2)) a)", "2");
 make_test ("(let ((x 0)) x x x)", "0");
+make_test ("(let ((a)) a)", "NIL");
+make_test ("(let* ((a)) a)", "NIL");
 make_test ("(flet ((a () (write \"\"))) (a))", "\"\"\n\"\"");
 make_test ("(labels ((a NIL (write \"\"))) (a))", "\"\"\n\"\"");
 make_test ("(labels ((a () (b)) (b () (write \"\"))) (a))", "\"\"\n\"\"");
 make_test ("(macrolet ((w (a) `(write ,a))) (w \"\"))", "\"\"\n\"\"");
 make_test ("(defconstant a 8)", "A");
+make_test ("(defconstant a 8)", "A");
 make_test ("a", "8");
 make_test ("(defparameter b 9)", "B");
 make_test ("(defvar b (0))", "B");
@@ -218,6 +236,11 @@ make_test ("(setf c (+ c 1))", "5");
 make_test ("c", "5");
 make_test ("(setq)", "NIL");
 make_test ("(setq b 9 c 4 c (+ c 1))", "5");
+make_test ("(psetq)", "NIL");
+make_test ("(psetq xx 0)", "NIL");
+make_test ("(psetq xx 1 yy xx)", "NIL");
+make_test ("xx", "1");
+make_test ("yy", "0");
 make_test ("(setf tt (list 'a 'b 'd))", "(A B D)");
 make_test ("(setf (car tt) 'f)", "F");
 make_test ("tt", "(F B D)");
@@ -232,6 +255,8 @@ make_test ("arr", "#(NIL NIL \"aaa\")");
 make_test ("(let ((b 10)) b)", "10");
 make_test ("(let ((b 1)) (setf b 2) b)", "2");
 make_test ("b", "9");
+make_test ("(let ((s \"aaa\")) (list (setf (elt s 1) #\\b) s))", "(#\\b \"aba\")");
+make_test ("(let ((l (list 'a 'b 'c))) (list (setf (elt l 2) 'd) l))", "(D (A B D))");
 make_test ("(let* ((x 1) (y x)) y)", "1");
 make_test ("(defun f ())", "F");
 make_test ("#'f", "#<FUNCTION F>");
@@ -264,9 +289,9 @@ make_test ("(g 1 2)", "2");
 make_test ("(defun g2 (&optional (x \"a\")) x)", "G2");
 make_test ("(g2)", "\"a\"");
 make_test ("(g2 4)", "4");
-make_test ("(defun g3 (&optional (x \"a\" provided)) provided)", "G3");
-make_test ("(g3)", "NIL");
-make_test ("(g3 7)", "T");
+make_test ("(defun g3 (&optional (x \"a\" provided)) (list x provided))", "G3");
+make_test ("(g3)", "(\"a\" NIL)");
+make_test ("(g3 7)", "(7 T)");
 make_test ("(defun g4 (x &optional y &rest z) z)", "G4");
 make_test ("(g4 1 2 3 (+ 3 1) 5)", "(3 4 5)");
 make_test ("(defun fun (&rest x) x)", "FUN");
@@ -278,9 +303,16 @@ make_test ("(fun2 :b 0)", "0");
 make_test ("(fun2 :a 0)", "NIL");
 make_test ("(fun2 :c 'tt :b (+ 1 2))", "3");
 make_test ("(fun2)", "NIL");
+make_test ("(defun fun2 (&key ((:k aa) 1 supp)) (list aa supp))", "FUN2");
+make_test ("(fun2)", "(1 NIL)");
+make_test ("(fun2 :k 1)", "(1 T)");
 make_test ("(defun fun2 (&rest r &key a b c) r)", "FUN2");
 make_test ("(fun2 :b (+ 1 2) :c 'tt)", "(:B 3 :C TT)");
 make_test ("(fun2)", "NIL");
+make_test ("(defun fun (x &rest y &key z) y)", "FUN");
+make_test ("(fun 1 :z 0)", "(:Z 0)");
+make_test ("(defun fun (x &optional y &rest z &key k) z)", "FUN");
+make_test ("(fun 0 1 :k 2)", "(:K 2)");
 make_test ("(let ((x 0)) (defun inc () (setf x (+ x 1))))", "INC");
 make_test ("(inc)", "1");
 make_test ("(inc)", "2");
@@ -293,6 +325,7 @@ make_test ("(input-stream-p *standard-input*)", "T");
 make_test ("(input-stream-p *standard-output*)", "NIL");
 make_test ("(output-stream-p *standard-input*)", "NIL");
 make_test ("(output-stream-p *standard-output*)", "T");
+make_test ("(interactive-stream-p *standard-input*)", "NIL");
 make_test ("(upper-case-p #\\A)", "T");
 make_test ("(upper-case-p #\\a)", "NIL");
 make_test ("(upper-case-p #\\2)", "NIL");
@@ -342,6 +375,9 @@ make_test ("(typep 'aaa 'keyword)", "NIL");
 make_test ("(typep nil 'boolean)", "T");
 make_test ("(typep #p\"\" 'pathname)", "T");
 make_test ("(typep (open #p\"README\") 'stream)", "T");
+make_test ("(typep (open #p\"README\") 'file-stream)", "T");
+make_test ("(typep (make-string-input-stream \"hello\") 'string-stream)", "T");
+make_test ("(deftype not-integer () `(not integer))", "NOT-INTEGER");
 make_test ("(make-string 3)", "\"\0\0\0\"");
 make_test ("(make-symbol \"aaa\")", "#:|aaa|");
 make_test ("(boundp 'b)", "T");
@@ -359,18 +395,26 @@ make_test ("(funcall (complement #'numberp) \"\")", "T");
 make_test ("(symbol-name 'aaa)", "\"AAA\"");
 make_test ("(symbol-name :bbb)", "\"BBB\"");
 make_test ("(symbol-package 'if)", "#<PACKAGE \"COMMON-LISP\">");
+make_test ("(symbol-package 'caar)", "#<PACKAGE \"COMMON-LISP\">");
 make_test ("(symbol-package 'ggg)", "#<PACKAGE \"COMMON-LISP-USER\">");
 make_test ("(symbol-package :eee)", "#<PACKAGE \"KEYWORD\">");
+make_test ("(symbol-package '#:aaa)", "NIL");
+make_test ("(symbol-package (make-symbol \"aaa\"))", "NIL");
 make_test ("(special-operator-p 'if)", "T");
 make_test ("(special-operator-p 'car)", "NIL");
 make_test ("(special-operator-p 'aaa)", "NIL");
 make_test ("(equal #\\a #\\a)", "T");
 make_test ("(equal 0 0)", "T");
 make_test ("(equal 0 0.0)", "NIL");
+make_test ("(equal (cons 1 2) 1)", "NIL");
 make_test ("(equal (cons 'a 'b) (cons 'a 'b))", "T");
+make_test ("(equal '(1 2 3) '(1 2 3))", "T");
+make_test ("(equal '(1 2 . 4) '(1 2 . 3))", "NIL");
 make_test ("(equal (make-array '(1 2 3)) (make-array '(1 2 3)))", "NIL");
 make_test ("(equalp 0 0)", "T");
 make_test ("(equalp 0 0.0)", "T");
+make_test ("(equalp '(1 2 3) '(1 2 3))", "T");
+make_test ("(equalp '(1 2 . 4) '(1 2 . 3))", "NIL");
 make_test ("(equalp (make-array '(1 2 3)) (make-array '(1 2 3)))", "T");
 make_test ("(string \"ccc\")", "\"ccc\"");
 make_test ("(string 'ddd)", "\"DDD\"");
@@ -448,6 +492,8 @@ make_test ("(list-length '(0 1 2 3))", "4");
 make_test ("(make-array 4)", "#(NIL NIL NIL NIL)");
 make_test ("(make-array '(1 2 3))", "#<ARRAY, RANK 3>");
 make_test ("(make-array nil)", "#<ARRAY, RANK 0>");
+make_test ("(vector)", "#()");
+make_test ("(vector 'a 'b \"\")", "#(A B \"\")");
 make_test ("(array-has-fill-pointer-p \"aaa\")", "NIL");
 make_test ("(array-rank #())", "1");
 make_test ("(array-rank (make-array nil))", "0");
@@ -467,17 +513,57 @@ make_test ("(array-row-major-index (make-array '(1 2 3)) 0 0 0)", "0");
 make_test ("(array-row-major-index (make-array '(2 3 4)) 0 0 1)", "1");
 make_test ("(array-row-major-index (make-array '(2 3 4)) 0 2 0)", "8");
 make_test ("(array-row-major-index (make-array '(2 3 4)) 1 0 0)", "12");
-make_test ("(make-hash-table)", "#<HASH-TABLE EQ 0/1024>");
-make_test ("(hash-table-size (make-hash-table))", "1024");
-make_test ("(hash-table-count (make-hash-table))", "0");
-make_test ("(gethash 10 (make-hash-table))", "NIL\nNIL");
-make_test ("(remhash '(1 2) (make-hash-table))", "NIL");
+make_test ("(defparameter tbl (make-hash-table))", "TBL");
+make_test ("tbl", "#<HASH-TABLE EQ 0/1024>");
+make_test ("(hash-table-size tbl)", "1024");
+make_test ("(hash-table-count tbl)", "0");
+make_test ("(hash-table-test tbl)", "EQ");
+make_test ("(gethash 'k tbl)", "NIL\nNIL");
+make_test ("(setf (gethash 'k tbl) 10)", "10");
+make_test ("(gethash 'k tbl)", "10\nT");
+make_test ("(remhash 'k tbl)", "T");
+make_test ("(gethash 'k tbl)", "NIL\nNIL");
 make_test ("(last '(1 2 3))", "(3)");
 make_test ("(last '(1 2 3) 0)", "NIL");
 make_test ("(last '(1 2 3) 2)", "(2 3)");
+make_test ("(read-line)\nabc", "\"abc\"\nNIL", 1);
 make_test ("(read-line (make-string-input-stream \"hello world\"))", "\"hello world\"\nT");
 make_test ("(read-line (make-string-input-stream \"hello world\n" .
 	   "\"))", "\"hello world\"\nNIL");
+make_test ("(read)\n\"hello\"\"world\" 12 `(1 2 3) #\\c aa  ", "\"hello\"", 1);
+make_test ("(read)", "\"world\"");
+make_test ("(read)", "12");
+make_test ("(read)", "`(1 2 3)");
+make_test ("(read)", "#\\c");
+make_test ("(read)", "AA");
+make_test ("(read-line)", "\" \"\nNIL");
+make_test ("(defparameter inp (make-string-input-stream \"\\\"hello\\\"\\\"world\\\" 12 `(1 2 3) #\\\\c aa  \"))", "INP");
+make_test ("(read inp)", "\"hello\"");
+make_test ("(read inp)", "\"world\"");
+make_test ("(read inp)", "12");
+make_test ("(read inp)", "`(1 2 3)");
+make_test ("(read inp)", "#\\c");
+make_test ("(read inp)", "AA");
+make_test ("(read-line inp)", "\" \"\nT");
+make_test ("(read (make-string-input-stream \"hello\"))", "HELLO");
+make_test ("(read (make-string-input-stream \"123\"))", "123");
+make_test ("(read-preserving-whitespace)\n\"hello\"\"world\" 12 `(1 2 3) #\\c aa  ", "\"hello\"", 1);
+make_test ("(read-preserving-whitespace)", "\"world\"");
+make_test ("(read-preserving-whitespace)", "12");
+make_test ("(read-preserving-whitespace)", "`(1 2 3)");
+make_test ("(read-preserving-whitespace)", "#\\c");
+make_test ("(read-preserving-whitespace)", "AA");
+make_test ("(read-line)", "\"  \"\nNIL");
+make_test ("(defparameter inp2 (make-string-input-stream \"\\\"hello\\\"\\\"world\\\" 12 `(1 2 3) #\\\\c aa  \"))", "INP2");
+make_test ("(read-preserving-whitespace inp2)", "\"hello\"");
+make_test ("(read-preserving-whitespace inp2)", "\"world\"");
+make_test ("(read-preserving-whitespace inp2)", "12");
+make_test ("(read-preserving-whitespace inp2)", "`(1 2 3)");
+make_test ("(read-preserving-whitespace inp2)", "#\\c");
+make_test ("(read-preserving-whitespace inp2)", "AA");
+make_test ("(read-line inp2)", "\"  \"\nT");
+make_test ("(read-preserving-whitespace (make-string-input-stream \"hello\"))", "HELLO");
+make_test ("(read-preserving-whitespace (make-string-input-stream \"123\"))", "123");
 make_test ("(eval '(write \"\"))", "\"\"\n\"\"");
 make_test ("(setq var 10)", "10");
 make_test ("(let ((var 12)) (eval 'var))", "10");
@@ -491,12 +577,14 @@ make_test ("(write-byte (char-code #\\a) *standard-output*)", "a97");
 make_test ("(fresh-line)", "NIL");
 make_test ("(terpri)", "\nNIL");
 make_test ("(write-line \"aaa\")", "aaa\n\"aaa\"");
+make_test ("(write-sequence \"hello world\" *standard-output* :start 1 :end 5)", "ello\n\"hello world\"");
 make_test ("(prin1 '|Aa,a|)", "|Aa,a|\n|Aa,a|");
 make_test ("(prin1 #\\a)", "#\\a\n#\\a");
-make_test ("(prin1 \"aaa\")", "\"aaa\"\n\"aaa\"");
+make_test ("(prin1 \"aaa\" *standard-output*)", "\"aaa\"\n\"aaa\"");
 make_test ("(princ '|Aa,a|)", "Aa,a\n|Aa,a|");
 make_test ("(princ #\\a)", "a\n#\\a");
-make_test ("(princ \"aaa\")", "aaa\n\"aaa\"");
+make_test ("(princ \"aaa\" *standard-output*)", "aaa\n\"aaa\"");
+make_test ("(print \"aaa\" *standard-output*)", "\n\"aaa\" \n\"aaa\"");
 make_test ("(format t \"aaa\")", "aaa\nNIL");
 make_test ("(format t \"aaa~~\")", "aaa~\nNIL");
 make_test ("(format t \"aaa~%~%\")", "aaa\n\nNIL");
@@ -518,10 +606,11 @@ make_test ("(make-string-input-stream \"aaa\")", "#<STREAM ?>");
 make_test ("(defparameter str (make-string-output-stream))", "STR");
 make_test ("(write 'aaa :stream str)", "AAA");
 make_test ("(write \"bbb\" :stream str)", "\"bbb\"");
+make_test ("(write 'ccc :stream str)", "CCC");
 make_test ("(write #\\c :stream str)", "#\\c");
 make_test ("(write 100 :stream str)", "100");
 make_test ("(write 1.2 :stream str)", "1.2");
-make_test ("(get-output-stream-string str)", "\"AAA\\\"bbb\\\"#\\\\c1001.2\"");
+make_test ("(get-output-stream-string str)", "\"AAA\\\"bbb\\\"CCC#\\\\c1001.2\"");
 make_test ("(get-output-stream-string str)", "\"\"");
 make_test ("(gensym)", "#:G1");
 make_test ("(gensym)", "#:G2");
@@ -545,6 +634,7 @@ make_test ("(tagbody\n" .
 	   "  (write \"3\"))", "\"1\"\"3\"\nNIL");
 make_test ("(tagbody 1 (go 3) 2 3 (go 4) 4 (write \"\"))", "\"\"\nNIL");
 make_test ("(tagbody 1 2 (tagbody 3 (go 4)) 4 (write \"\"))", "\"\"\nNIL");
+make_test ("(tagbody (write 1) (block nil (write 2) (go c)) c (write 4))", "124\nNIL");
 make_test ("(block test (write 1) (write 2) (return-from test 10) (write 3))", "12\n10");
 make_test ("(block test (write 1) (block test2 (write 2) (return-from test 10) (write 3)))", "12\n10");
 make_test ("(block test (write 1) (block test2 (write 2) (return-from test (values 10 11)) (write 3)))", "12\n10\n11");
@@ -562,18 +652,38 @@ make_test ("(decf c)", "8");
 make_test ("(decf c c)", "0");
 make_test ("(and)", "T");
 make_test ("(and t)", "T");
+make_test ("(and (+ 1 2))", "3");
 make_test ("(and t t t)", "T");
 make_test ("(and t nil t)", "NIL");
 make_test ("(and (= 1 1) t)", "T");
 make_test ("(and t (= 1 2) t)", "NIL");
+make_test ("(and 1 2 (+ 1 2))", "3");
+make_test ("(and 0 (values 1 2))", "1\n2");
 make_test ("(or)", "NIL");
+make_test ("(or (+ 1 2))", "3");
 make_test ("(or nil nil nil)", "NIL");
 make_test ("(or (= 1 2) t)", "T");
 make_test ("(or t nil t)", "T");
+make_test ("(or nil nil (+ 1 2))", "3");
+make_test ("(or nil nil (+ 1 2) nil)", "3");
+make_test ("(or (values 1 2))", "1\n2");
+make_test ("(or (values 1 2) nil)", "1");
 make_test ("(concatenate 'string)", "\"\"");
 make_test ("(concatenate 'string \"\")", "\"\"");
 make_test ("(concatenate 'string \"aa\" \"bb\")", "\"aabb\"");
+make_test ("(do (x (y 0) (z 0 (1+ z))) ((>= z 10) (values x y z)) (write z))", "0123456789\nNIL\n0\n10");
+make_test ("(do ((x 0 (1+ x)) (y 0 (1+ x))) ((>= y 10)) (write y))", "0123456789\nNIL");
+make_test ("(do ((dd 0 (1+ dd))) (nil) (if (> dd 5) (return 10) (write dd)))", "012345\n10");
+make_test ("(do* (x (y 0) (z 0 (1+ z))) ((>= z 10) (values x y z)) (write z))", "0123456789\nNIL\n0\n10");
+make_test ("(do* ((x 0 (1+ x)) (y x (1+ x))) ((>= y 10)) (write y))", "023456789\nNIL");
+make_test ("(do* ((dd 0 (1+ dd))) (nil) (if (> dd 5) (return 10) (write dd)))", "012345\n10");
+make_test ("(dotimes (i 0 i))", "0");
 make_test ("(dotimes (tmp 10 tmp))", "10");
+make_test ("(dotimes (i 10) (write i) (if (= i 5) (return (values 11 12))))", "012345\n11\n12");
+make_test ("(dolist (i nil) (write i))", "NIL");
+make_test ("(dolist (i nil (values 0 1)))", "0\n1");
+make_test ("(dolist (i '(0 1 2 3)) (write i))", "0123\nNIL");
+make_test ("(dolist (i '(0 1 2 3)) (if (= i 2) (return (values 12 13))) (write i))", "01\n12\n13");
 make_test ("(mapcar #'car '((1 a) (2 b) (3 c)))", "(1 2 3)");
 make_test ("(mapcar #'abs '(3 -4 2 -5 -6))", "(3 4 2 5 6)");
 make_test ("(mapcar #'cons '(a b c) '(1 2 3))", "((A . 1) (B . 2) (C . 3))");
@@ -589,6 +699,13 @@ make_test ("(nreverse \"hello\")", "\"olleh\"");
 make_test ("(nreverse #(0 1 2 3))", "#(3 2 1 0)");
 make_test ("(adjoin 0 '(0 1 2))", "(0 1 2)");
 make_test ("(adjoin 0 '(1 2))", "(0 1 2)");
+make_test ("(fill \"abcdef\" #\\g)", "\"gggggg\"");
+make_test ("(fill '(0 1 2 3 4 5) 0 :start 1 :end 4)", "(0 0 0 0 4 5)");
+make_test ("(defparameter ll NIL)", "LL");
+make_test ("(push (1+ 0) ll)", "(1)");
+make_test ("ll", "(1)");
+make_test ("(pop ll)", "1");
+make_test ("ll", "NIL");
 make_test ("(cond (t (+ 1 2)))", "3");
 make_test ("(cond (nil 2) (t 3) (nil 4))", "3");
 make_test ("(every #'evenp '(0 2 4))", "T");
@@ -634,10 +751,31 @@ make_test ("(package-used-by-list (symbol-package 'if))", "(#<PACKAGE \"COMMON-L
 make_test ("(list-all-packages)", "(#<PACKAGE \"COMMON-LISP-USER\"> #<PACKAGE \"COMMON-LISP\"> #<PACKAGE \"KEYWORD\">)");
 make_test ("(make-package \"test\")", "#<PACKAGE \"test\">");
 make_test ("(list-all-packages)", "(#<PACKAGE \"test\"> #<PACKAGE \"COMMON-LISP-USER\"> #<PACKAGE \"COMMON-LISP\"> #<PACKAGE \"KEYWORD\">)");
+make_test ("(rename-package \"test\" \"TEST\" '(nick1 #\\n \"nick3\"))", "#<PACKAGE \"TEST\">");
+make_test ("(list-all-packages)", "(#<PACKAGE \"TEST\"> #<PACKAGE \"COMMON-LISP-USER\"> #<PACKAGE \"COMMON-LISP\"> #<PACKAGE \"KEYWORD\">)");
+make_test ("(package-nicknames \"TEST\")", "(\"NICK1\" \"n\" \"nick3\")");
 make_test ("(in-package \"CL\")", "#<PACKAGE \"COMMON-LISP\">");
 make_test ("*package*", "#<PACKAGE \"COMMON-LISP\">");
 make_test ("(in-package cl-user)", "#<PACKAGE \"COMMON-LISP-USER\">");
 make_test ("*package*", "#<PACKAGE \"COMMON-LISP-USER\">");
+make_test ("(use-package 'cl)", "T");
+make_test ("(make-package \"newtest\")", "#<PACKAGE \"newtest\">");
+make_test ("(in-package \"newtest\")", "#<PACKAGE \"newtest\">");
+make_test ("(cl:package-use-list cl:*package*)", "COMMON-LISP:NIL");
+make_test ("(cl:use-package 'cl)", "T");
+make_test ("(cl:package-use-list cl:*package*)", "(#<PACKAGE \"COMMON-LISP\">)");
+make_test ("(cadr '(0 1))", "1");
+make_test ("(cl:import 'cl:car)", "T");
+make_test ("(cl:import '(cl:car) '|newtest|)", "T");
+make_test ("(cl:export 'jjj)", "T");
+make_test ("(cl:export '(jjj) cl:*package*)", "T");
+make_test ("'|newtest|:jjj", "JJJ");
+make_test ("(cl:unexport '(jjj) cl:*package*)", "T");
+make_test ("(car ())", "NIL");
+make_test ("(cl:unuse-package 'cl)", "COMMON-LISP:T");
+make_test ("(cl:package-use-list cl:*package*)", "COMMON-LISP:NIL");
+make_test ("(car ())", "COMMON-LISP:NIL");
+make_test ("(cl:in-package cl-user)", "#<PACKAGE \"COMMON-LISP-USER\">");
 
 
 # arithmetic tests
@@ -759,6 +897,18 @@ waitpid ($pid, 0);
 
 sub make_test
 {
+    my $skip_only_first_line;
+    my $i = 0;
+
+    if (not defined ($_[2]))
+    {
+	$skip_only_first_line = 0;
+    }
+    else
+    {
+	$skip_only_first_line = $_[2];
+    }
+
     print $_[0] . " -> ";
 
     my $in = $_[0] . "\n";
@@ -769,7 +919,12 @@ sub make_test
     {
 	print $al_in $l . "\n";
 
-	<$al_out>;
+	if ($i == 0 or $skip_only_first_line == 0)
+	{
+	    <$al_out>;
+	}
+
+	$i++;
     }
 
     my @expected_outlines = split("\n", $_[1]);
@@ -778,7 +933,7 @@ sub make_test
 
     my $out;
 
-    for (my $i = 0; $i < scalar (@expected_outlines); $i++)
+    for ($i = 0; $i < scalar (@expected_outlines); $i++)
     {
 	$out = <$al_out>;
 
